from setuptools import setup, Extension, find_packages

package_name = 'internal_rpath'
setup(
    name=package_name,
    version='1.0',
    description='Auditwheel multiple top-level extensions example',
    package_data={package_name: ['liba.so']},
    packages=find_packages(),
    ext_modules=[
        Extension(
            f'{package_name}.example_a',
            ['src/example_a.pyx'],
            include_dirs=['lib-src/a'],
            library_dirs=[package_name],
            libraries=['a'],
            extra_link_args=['-Wl,-rpath,$ORIGIN'],
        ),
        Extension(
            f'{package_name}.example_b',
            ['src/example_b.pyx'],
            include_dirs=['lib-src/b'],
            library_dirs=['lib-src/b'],
            libraries=['b'],
        ),
    ],
)



import sys
import json
import platform as _platform_module
from collections import defaultdict
from typing import Dict, List, Optional, Set
from os.path import join, dirname, abspath
import logging


logger = logging.getLogger(__name__)

# https://docs.python.org/3/library/platform.html#platform.architecture
bits = 8 * (8 if sys.maxsize > 2 ** 32 else 4)


def get_arch_name() -> str:
    machine = _platform_module.machine()
    if machine not in {'x86_64', 'i686'}:
        return machine
    else:
        return {64: 'x86_64', 32: 'i686'}[bits]


_ARCH_NAME = get_arch_name()


def _validate_pep600_compliance(policies) -> None:
    symbol_versions: Dict[str, Dict[str, Set[str]]] = {}
    lib_whitelist: Set[str] = set()
    for policy in sorted(policies, key=lambda x: x['priority'], reverse=True):
        if policy['name'] == 'linux':
            continue
        if not lib_whitelist.issubset(set(policy['lib_whitelist'])):
            diff = lib_whitelist - set(policy["lib_whitelist"])
            raise ValueError(
                'Invalid "policy.json" file. Missing whitelist libraries in '
                f'"{policy["name"]}" compared to previous policies: {diff}'
            )
        lib_whitelist.update(policy['lib_whitelist'])
        for arch in policy['symbol_versions'].keys():
            symbol_versions_arch = symbol_versions.get(arch, defaultdict(set))
            for prefix in policy['symbol_versions'][arch].keys():
                policy_symbol_versions = set(
                    policy['symbol_versions'][arch][prefix])
                if not symbol_versions_arch[prefix].issubset(
                        policy_symbol_versions):
                    diff = symbol_versions_arch[prefix] - \
                           policy_symbol_versions
                    raise ValueError(
                        'Invalid "policy.json" file. Symbol versions missing '
                        f'in "{policy["name"]}_{arch}" for "{prefix}" '
                        f'compared to previous policies: {diff}'
                    )
                symbol_versions_arch[prefix].update(
                    policy['symbol_versions'][arch][prefix])
            symbol_versions[arch] = symbol_versions_arch


with open(join(dirname(abspath(__file__)), 'policy.json')) as f:
    _POLICIES = []
    _policies_temp = json.load(f)
    _validate_pep600_compliance(_policies_temp)
    for _p in _policies_temp:
        if _ARCH_NAME in _p['symbol_versions'].keys() or _p['name'] == 'linux':
            if _p['name'] != 'linux':
                _p['symbol_versions'] = _p['symbol_versions'][_ARCH_NAME]
            _p['name'] = _p['name'] + '_' + _ARCH_NAME
            _p['aliases'] = [alias + '_' + _ARCH_NAME
                             for alias in _p['aliases']]
            _POLICIES.append(_p)

POLICY_PRIORITY_HIGHEST = max(p['priority'] for p in _POLICIES)
POLICY_PRIORITY_LOWEST = min(p['priority'] for p in _POLICIES)


def load_policies():
    return _POLICIES


def _load_policy_schema():
    with open(join(dirname(abspath(__file__)), 'policy-schema.json')) as f:
        schema = json.load(f)
    return schema


def get_policy_by_name(name: str) -> Optional[Dict]:
    matches = [p for p in _POLICIES
               if p['name'] == name or name in p['aliases']]
    if len(matches) == 0:
        return None
    if len(matches) > 1:
        raise RuntimeError('Internal error. Policies should be unique')
    return matches[0]


def get_policy_name(priority: int) -> Optional[str]:
    matches = [p['name'] for p in _POLICIES if p['priority'] == priority]
    if len(matches) == 0:
        return None
    if len(matches) > 1:
        raise RuntimeError('Internal error. priorities should be unique')
    return matches[0]


def get_priority_by_name(name: str) -> Optional[int]:
    policy = get_policy_by_name(name)
    return None if policy is None else policy['priority']


def get_replace_platforms(name: str) -> List[str]:
    """Extract platform tag replacement rules from policy

    >>> get_replace_platforms('linux_x86_64')
    []
    >>> get_replace_platforms('linux_i686')
    []
    >>> get_replace_platforms('manylinux1_x86_64')
    ['linux_x86_64']
    >>> get_replace_platforms('manylinux1_i686')
    ['linux_i686']

    """
    if name.startswith('linux'):
        return []
    if name.startswith('manylinux_'):
        return ['linux_' + '_'.join(name.split('_')[3:])]
    return ['linux_' + '_'.join(name.split('_')[1:])]


# These have to be imported here to avoid a circular import.
from .external_references import lddtree_external_references  # noqa
from .versioned_symbols import versioned_symbols_policy  # noqa

__all__ = ['lddtree_external_references', 'versioned_symbols_policy',
           'load_policies', 'POLICY_PRIORITY_HIGHEST',
           'POLICY_PRIORITY_LOWEST']



from unittest.mock import patch

import pytest

from auditwheel.policy import get_arch_name, get_policy_name, \
    get_priority_by_name, get_replace_platforms, _validate_pep600_compliance


@patch("auditwheel.policy._platform_module.machine")
@patch("auditwheel.policy.bits", 32)
@pytest.mark.parametrize("reported_arch,expected_arch", [
    ("armv6l", "armv6l"),
    ("armv7l", "armv7l"),
    ("i686", "i686"),
    ("x86_64", "i686"),
])
def test_32bits_arch_name(machine_mock, reported_arch, expected_arch):
    machine_mock.return_value = reported_arch
    machine = get_arch_name()
    assert machine == expected_arch


@patch("auditwheel.policy._platform_module.machine")
@patch("auditwheel.policy.bits", 64)
@pytest.mark.parametrize("reported_arch,expected_arch", [
    ("aarch64", "aarch64"),
    ("ppc64le", "ppc64le"),
    ("x86_64", "x86_64"),
])
def test_64bits_arch_name(machine_mock, reported_arch, expected_arch):
    machine_mock.return_value = reported_arch
    machine = get_arch_name()
    assert machine == expected_arch


@pytest.mark.parametrize("name,expected", [
    ("linux_aarch64", []),
    ("manylinux1_ppc64le", ["linux_ppc64le"]),
    ("manylinux2014_x86_64", ["linux_x86_64"]),
    ("manylinux_2_24_x86_64", ["linux_x86_64"]),
])
def test_replacement_platform(name, expected):
    assert get_replace_platforms(name) == expected


def test_pep600_compliance():
    _validate_pep600_compliance([{
        "name": "manylinux1", "priority": 100, "symbol_versions": {
            "i686": {"CXXABI": ["1.3"]},
        },
        "lib_whitelist": ["libgcc_s.so.1"]
    }, {
        "name": "manylinux2010", "priority": 90, "symbol_versions": {
            "i686": {"CXXABI": ["1.3", "1.3.1"]},
        },
        "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
    }])

    _validate_pep600_compliance([{
        "name": "manylinux1", "priority": 100, "symbol_versions": {
            "i686": {"CXXABI": ["1.3"]},
            "x86_64": {"CXXABI": ["1.3"]},
        },
        "lib_whitelist": ["libgcc_s.so.1"]
    }, {
        "name": "manylinux2010", "priority": 90, "symbol_versions": {
            "i686": {"CXXABI": ["1.3", "1.3.1"]},
        },
        "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
    }])

    with pytest.raises(ValueError, match=r"manylinux2010_i686.*CXXABI.*1.3.2"):
        _validate_pep600_compliance([{
            "name": "manylinux1", "priority": 100, "symbol_versions": {
                "i686": {"CXXABI": ["1.3", "1.3.2"]},
            },
            "lib_whitelist": ["libgcc_s.so.1"]
        }, {
            "name": "manylinux2010", "priority": 90, "symbol_versions": {
                "i686": {"CXXABI": ["1.3", "1.3.1"]},
            },
            "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
        }])

    with pytest.raises(ValueError, match=r"manylinux2010.*libstdc\+\+\.so\.6"):
        _validate_pep600_compliance([{
            "name": "manylinux1", "priority": 100, "symbol_versions": {
                "i686": {"CXXABI": ["1.3"]},
            },
            "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"]
        }, {
            "name": "manylinux2010", "priority": 90, "symbol_versions": {
                "i686": {"CXXABI": ["1.3", "1.3.1"]},
            },
            "lib_whitelist": ["libgcc_s.so.1"],
        }])


class TestPolicyAccess:

    def test_get_by_priority(self):
        _arch = get_arch_name()
        assert get_policy_name(65) == f'manylinux_2_27_{_arch}'
        assert get_policy_name(70) == f'manylinux_2_24_{_arch}'
        assert get_policy_name(80) == f'manylinux_2_17_{_arch}'
        if _arch in {'x86_64', 'i686'}:
            assert get_policy_name(90) == f'manylinux_2_12_{_arch}'
            assert get_policy_name(100) == f'manylinux_2_5_{_arch}'
        assert get_policy_name(0) == f'linux_{_arch}'

    def test_get_by_priority_missing(self):
        assert get_policy_name(101) is None

    @patch("auditwheel.policy._POLICIES", [
        {"name": "duplicate", "priority": 0},
        {"name": "duplicate", "priority": 0},
    ])
    def test_get_by_priority_duplicate(self):
        with pytest.raises(RuntimeError):
            get_policy_name(0)

    def test_get_by_name(self):
        _arch = get_arch_name()
        assert get_priority_by_name(f"manylinux_2_27_{_arch}") == 65
        assert get_priority_by_name(f"manylinux_2_24_{_arch}") == 70
        assert get_priority_by_name(f"manylinux2014_{_arch}") == 80
        assert get_priority_by_name(f"manylinux_2_17_{_arch}") == 80
        if _arch in {'x86_64', 'i686'}:
            assert get_priority_by_name(f"manylinux2010_{_arch}") == 90
            assert get_priority_by_name(f"manylinux_2_12_{_arch}") == 90
            assert get_priority_by_name(f"manylinux1_{_arch}") == 100
            assert get_priority_by_name(f"manylinux_2_5_{_arch}") == 100

    def test_get_by_name_missing(self):
        assert get_priority_by_name("nosuchpolicy") is None

    @patch("auditwheel.policy._POLICIES", [
        {"name": "duplicate", "priority": 0},
        {"name": "duplicate", "priority": 0},
    ])
    def test_get_by_name_duplicate(self):
        with pytest.raises(RuntimeError):
            get_priority_by_name("duplicate")




import platform
from pathlib import Path
import pytest
from auditwheel.wheel_abi import analyze_wheel_abi


HERE = Path(__file__).parent.resolve()


@pytest.mark.skipif(platform.machine() != 'x86_64', reason='only supported on x86_64')
@pytest.mark.parametrize('file, external_libs', [
    ('cffi-1.5.0-cp27-none-linux_x86_64.whl', {'libffi.so.5'}),
    ('python_snappy-0.5.2-pp260-pypy_41-linux_x86_64.whl', {'libsnappy.so.1'}),
])
def test_analyze_wheel_abi(file, external_libs):
    winfo = analyze_wheel_abi(str(HERE / file))
    assert set(winfo.external_refs['manylinux_2_5_x86_64']['libs']) == external_libs


@pytest.mark.skipif(platform.machine() != 'x86_64', reason='only supported on x86_64')
def test_analyze_wheel_abi_pyfpe():
    winfo = analyze_wheel_abi(str(HERE / 'fpewheel-0.0.0-cp35-cp35m-linux_x86_64.whl'))
    assert winfo.sym_tag == 'manylinux_2_5_x86_64'  # for external symbols, it could get manylinux1
    assert winfo.pyfpe_tag == 'linux_x86_64'        # but for having the pyfpe reference, it gets just linux



import os
from subprocess import CalledProcessError
from unittest.mock import patch, call

import pytest

from auditwheel.patcher import Patchelf


@patch("auditwheel.patcher.find_executable")
def test_patchelf_unavailable(find_executable):
    find_executable.return_value = False
    with pytest.raises(ValueError):
        Patchelf()


@patch("auditwheel.patcher.check_output")
def test_patchelf_check_output_fail(check_output):
    check_output.side_effect = CalledProcessError(1, "patchelf --version")
    with pytest.raises(ValueError, match="Could not call"):
        Patchelf()


@patch("auditwheel.patcher.check_output")
@pytest.mark.parametrize("version", ["0.9", "0.9.1", "0.10"])
def test_patchelf_version_check(check_output, version):
    check_output.return_value.decode.return_value = f"patchelf {version}"
    Patchelf()


@patch("auditwheel.patcher.check_output")
@pytest.mark.parametrize("version", ["0.8", "0.8.1", "0.1"])
def test_patchelf_version_check_fail(check_output, version):
    check_output.return_value.decode.return_value = f"patchelf {version}"
    with pytest.raises(ValueError, match=f"patchelf {version} found"):
        Patchelf()


@patch("auditwheel.patcher._verify_patchelf")
@patch("auditwheel.patcher.check_output")
@patch("auditwheel.patcher.check_call")
class TestPatchElf:
    """"Validate that patchelf is invoked with the correct arguments."""

    def test_replace_needed(self, check_call, _0, _1):
        patcher = Patchelf()
        filename = "test.so"
        soname_old = "TEST_OLD"
        soname_new = "TEST_NEW"
        patcher.replace_needed(filename, soname_old, soname_new)
        check_call.assert_called_once_with(['patchelf', '--replace-needed',
                                            soname_old, soname_new, filename])

    def test_set_soname(self, check_call, _0, _1):
        patcher = Patchelf()
        filename = "test.so"
        soname_new = "TEST_NEW"
        patcher.set_soname(filename, soname_new)
        check_call.assert_called_once_with(['patchelf', '--set-soname',
                                            soname_new, filename])

    def test_set_rpath(self, check_call, _0, _1):
        patcher = Patchelf()
        patcher.set_rpath("test.so", "$ORIGIN/.lib")
        check_call_expected_args = [call(['patchelf', '--remove-rpath',
                                          'test.so']),
                                    call(['patchelf', '--force-rpath',
                                          '--set-rpath', '$ORIGIN/.lib',
                                          'test.so'])]

        assert check_call.call_args_list == check_call_expected_args

    def test_get_rpath(self, _0, check_output, _1):
        patcher = Patchelf()
        check_output.return_value = b"existing_rpath"
        result = patcher.get_rpath("test.so")
        check_output_expected_args = [call(['patchelf', '--print-rpath',
                                            'test.so'])]

        assert result == check_output.return_value.decode()
        assert check_output.call_args_list == check_output_expected_args
